/***************************************************************************
 *   Copyright (C) 2005 by Krzysztof Zawadyl                               *
 *   k.zawadyl@gmail.com                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "configure.h"
#include "metamonitor.h"

#include <qradiobutton.h>
#include <qcheckbox.h>
#include <qlineedit.h>

#include <kurlrequester.h>
#include <knuminput.h>

/**
 * @brief Constructor
 * 
 * Sets the configure window defaults
 * 
 * @param parent pointer to MetaMonitor
 * @param name window object name
 */
Configure::Configure( MetaMonitor *parent, const char *name ) : ConfigureBase( parent, name )
{
	mUpdateInterval->setValue( parent->updateInterval()/1000 );
	mPopupsLifetime->setValue( parent->popupsLifetime()/1000 );
	
	QString fileName = parent->getLogFile();
	
	if( fileName == "/var/log/messages" )
		mRadio1->setChecked( true );
	else
	if( fileName == "/var/log/everything/current" )
		mRadio2->setChecked( true );
	else
	{
		mRadio3->setChecked( true );
		customFile->setURL( fileName );
	}

	if( parent->getDrawShadow() )
		mCheckBox1->setChecked( true );

	if( parent->getGroupMessages() )
		groupMessagesBox->setChecked( true );
	
	switch( parent->useDefaultRegExp() )
	{
	case MetaMonitor::MetaLog:
		defaultRegExpMetaLogRadio->setChecked( true );
		break;
	case MetaMonitor::SysLog:
		defaultRegExpSysLogRadio->setChecked( true );
		break;
	default:
		customRegExpRadio->setChecked( true );
		break;
	}
	
	customRegExpEdit->setText( parent->getCustomRegExp() );
}


/**
 * @brief Descructor
 */
Configure::~Configure()
{
}

/**
 * @brief Returns an update interval
 * 
 * @return interval in seconds
 */
unsigned int Configure::updateInterval() const
{
	return (int)(mUpdateInterval->value()*1000);
}

/**
 * @brief Returns popup's lifetime
 * 
 * @return lifetime in seconds
 */
unsigned int Configure::popupsLifetime() const
{
	return (int)(mPopupsLifetime->value()*1000);
}

/**
 * @brief Returns a file to monitor
 * @return file name
 */
QString Configure::getFileName()
{
	QString str;
	
	if( mRadio1->isChecked() )
		str = "/var/log/messages";
	else
	if( mRadio2->isChecked() )
		str = "/var/log/everything/current";
	else
		str = customFile->url();
	
	return str;
}

/**
 * @brief Returns if the 'draw shadow' checkbox is selected
 * @return mCheckBox1 state
 */
bool Configure::drawShadow()
{
	return mCheckBox1->isChecked();
}

/**
 * @brief Returns if the 'group messages' checkbox is selected
 * @return groupMessagesBox state
 */
bool Configure::groupMessages()
{
	return groupMessagesBox->isChecked();
}

/**
 * @brief Returns if the Use default regexp Radio is checked
 * 
 * It also checks if the Custom regexp Lineedit is filled with some text
 * 
 * @return use Default RegExp Radio state
 */
MetaMonitor::LogDaemonType Configure::useDefaultRegExp()
{
	if( defaultRegExpMetaLogRadio->isChecked() )
		return MetaMonitor::MetaLog;
	else
	if( defaultRegExpSysLogRadio->isChecked() )
	    return MetaMonitor::SysLog;
	
	return MetaMonitor::NoLog;
}

/**
 * @brief Returns the custom RegExp String
 * @return the custom RegExp String
 */
QString Configure::getCustomRegExp()
{
	return customRegExpEdit->text();
}
#include "configure.moc"
