/***************************************************************************
 *   Copyright (C) 2005 by Krzysztof Zawadyl                               *
 *   k.zawadyl@gmail.com                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "filemonitor.h"
#include "message.h"

#include <qfile.h>
#include <qptrlist.h>
#include <qtimer.h>
#include <qregexp.h>

#include <kmessagebox.h>
#include <klocale.h>

/**
 * @brief Constructor
 * 
 * Creates a monitor for a single file.
 * 
 * @param p pointer to the parent widget
 * @param fName file name
 */
FileMonitor::FileMonitor( MetaMonitor *p, const QString &fName )
{
	parent = p;
	mFile = new QFile( fName );
	init(false);
	mSize = mFile->size();
	mHandle = mFile->handle();
}

/**
 * @brief Destructor
 * 
 * Closes monitored filemonitor
 */
FileMonitor::~FileMonitor()
{
	if( mFile )
		delete mFile;
}

/**
 * @brief monitor's initialisation 
 * 
 * Opens a file and goes to the end of it. Pops an error message if fails
 *
 * @param quiet if TRUE, the FileMonitor will popup the MessageBox when error 
 * @return TRUE if success, FALSE if not
 */
bool FileMonitor::init(bool quiet)
{
	if( !mFile->isOpen() )
		mFile->open( IO_ReadOnly );
	
	if( mFile->isOpen() )
	{
		// go to the end of file
		mFile->at( mFile->size() );
		
		return TRUE;
	}
	
	if( !quiet )
		KMessageBox::detailedError( 
	                            parent, 
	                            i18n( "Cannot open a file" ), 
	                            i18n( "%1: %2" ).arg( mFile->name(), mFile->errorString() ), 
	                            i18n( "Error" ) );
	return FALSE;
}

/**
 * @brief Message's fetching
 * 
 * Fetches new messages and puts'em to the list
 *
 * @param mList list of messages to store the new message object
 * @return count of new messages
 */
int FileMonitor::fetchMessages( QPtrList<Message> *mList )
{
	// if the file dissapear, try to look for it
	// the restarted syslog daemon should create 
	// it in case it was deleted
	if( !mFile->exists() || !mFile->isOpen() )
	{
		if( !init(true) )
			return 0;
	}
	
	int count=0;
	
	// if the file is now smaller than last time
	// we reopen the filemonitor
	// this can happen when log daemon rotates the log files
	if( !mFile->exists() || mFile->size() < mSize || mFile->handle() != mHandle )
	{
		mFile->close();
		if( init(false) )
		{
			mList->append( new Message("MetaMonitor", "ReOpening log file") );
			count++;
		}
		else
			return 0;
	}
	
	// remember the current file size and handle
	mSize = mFile->size();
	mHandle = mFile->handle();
	
	mFile->flush();
	// messages fetching...
	int len;
	QString line;
	do 
	{
		len = mFile->readLine( line, 4096 );
		if( len>0 && !line.isEmpty() && line!="\n" && line!="\r\n" ) 
		{
			QString from, mess, time = "";
			
			if( line.contains( "Last output repeated" ) )
			{
				from = i18n( "Log daemon" );
				mess = line.stripWhiteSpace();
			}
			else
			{
				// parsing (not good, cause we just trust the file)
				// every line in syslog/metalog should look like this:
				// <date> [<sender>] <text>
				QRegExp rx;
				if( parent->useDefaultRegExp() == MetaMonitor::MetaLog )
					rx.setPattern( "^([^:]+(?::\\d{2}){2})\\s\\[([^\\[]+)\\](.*)$" );
				else
				if( parent->useDefaultRegExp() == MetaMonitor::SysLog )
					rx.setPattern( "^([^:]+(?::\\d{2}){2})\\s*(\\S*|\\S*\\s\\S*:)\\s*(.*)$" );
				else
					rx.setPattern( parent->getCustomRegExp() );
				
				rx.search( line );
				time = rx.cap( 1 );
				from = rx.cap( 2 );
				
				// get rid of trash
				if( from.right(1) == ":" )
					from.truncate( from.length()-1 );
				
				mess = rx.cap( 3 ).stripWhiteSpace();
				
			}
			
			Message *m = new Message( from, mess );
			
			// we better don't set the date from log (parsing and locale problems)
			// just set the date to message's default (current dateTime)
			/* if( time != "" )
			m->setDate( time ); */
			
			mList->append( m );
			count++;
		}
	} 
	while( len > 0 );
	
	return count;
}

#include "filemonitor.moc"
