/***************************************************************************
 *   Copyright (C) 2005 by Krzysztof Zawadyl                               *
 *   k.zawadyl@gmail.com                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "metamonitor.h"
#include "filemonitor.h"
#include "message.h"
#include "messagetable.h"
#include "popup.h"
#include "configure.h"
#include "dcoplistener.h"

#include <qtimer.h>
#include <qpixmap.h>
#include <qcursor.h>

#include <ksystemtray.h>
#include <kpopupmenu.h>
#include <kaction.h>
#include <kmessagebox.h>
#include <klocale.h>
#include <kapp.h>
#include <kaboutapplication.h>
#include <kstandarddirs.h>
#include <kglobal.h>
#include <kconfig.h>


/**
 * @brief MetaMonitor's constructor
 * Creates application, applies settings and so on :)
 */
MetaMonitor::MetaMonitor() : KSystemTray()
{
	listener = new DCOPListener( this );
	mIcon = loadIcon( "metamonitor" );
	setPixmap( mIcon );
	
	// load config
	KConfig cfg( KGlobal::dirs()->localkdedir()+"/share/config/metamonitorrc", true );
	drawShadow 		= cfg.readBoolEntry( "DrawShadow", true );
	groupMessages	= cfg.readBoolEntry( "GroupMessages", true );
	pausePopups		= cfg.readBoolEntry( "PausePopups", false );
	logFile 		= cfg.readEntry( "LogFile", "/var/log/everything/current" );
	mUpdateInterval = cfg.readNumEntry( "UpdateInterval", 2000 );
	mPopupsLifetime = cfg.readNumEntry( "PopupsLifetime", 4000 );
	defaultRegExp   = (LogDaemonType)cfg.readNumEntry( "DefaultRegExpType", MetaLog );
	customRegExp    = cfg.readEntry( "CustomRegExp", "" );
	
	// Context menu
	KPopupMenu* menu = contextMenu();
	menu->setCheckable( TRUE );
	menu->insertItem( i18n( "&View logs" ), this, SLOT( messageTable() ) );
	KAction* configure = KStdAction::preferences( this, SLOT( configure() ), actionCollection(), "configure" );
	configure->plug( menu );
	pauseId = menu->insertItem( i18n( "&Pause popups" ), this, SLOT( switchPausePopups() ) );
	menu->setItemChecked( pauseId, getPausePopups() );
	menu->insertSeparator();
	menu->insertItem( i18n("&About"), this, SLOT( about() ) );
	connect( this, SIGNAL( quitSelected() ), kapp, SLOT( quit(void) ) );
	messageViewer = new MessageTable( this );
	
	pop = new Popup(this);
	
	messagesList.clear();
	
	// initialize log file to watch
	fileMonitor = new FileMonitor( this, logFile );

	// start timer
	mTimer = new QTimer( this, "timer" );
	connect( mTimer, SIGNAL( timeout() ), this, SLOT( update(void) ) );
	mTimer->start( (int)mUpdateInterval );
}

/**
 * @brief MetaMonitor's descructor
 * Closes timer, file and popup
 */
MetaMonitor::~MetaMonitor()
{
	if( mTimer )
		delete mTimer;
	
	if( fileMonitor )
		delete fileMonitor;
	
	if( pop )
		delete pop;
}

/**
 * @brief mouse click on tray-icon reaction
 *
 * If it's left button - show the messages list
 * If right - show the context menu
 */
void MetaMonitor::mousePressEvent(QMouseEvent* ev)
{
	if( ev->button() == Qt::RightButton )
		contextMenu()->exec( QCursor::pos() );
	else 
	if( ev->button() == Qt::LeftButton )
		messageTable();
}

/**
 * @brief Pause/unpause popup messaging
 * 
 * It saves the configuration after changing the pausePopups' value
 */
void MetaMonitor::switchPausePopups()
{
	pausePopups = !getPausePopups();
	contextMenu()->setItemChecked( pauseId, pausePopups );
	writeConfig();
}

/**
 * @brief show/hide the message list window
 */
void MetaMonitor::messageTable()
{
	messageViewer->setHidden( messageViewer->isVisible() );
}

/**
 * @brief checks if there are some new messages
 * 
 * If there are some new messages they are stored in the messageTable window
 * and the popup is created (depends of pausePopups' state)
 */
void MetaMonitor::update()
{
	if( fileMonitor && fileMonitor->fetchMessages(&messagesList) )
	{
		bool pause = getPausePopups();
		while( !messagesList.isEmpty() )
		{
			Message *m = messagesList.take( 0 );
			messageViewer->addMessage( m );
			
			if( !pause )
			if( getGroupMessages() )
				pop->addToMessageQueue( m );
			else
				Popup::message( this, m );
			delete m;
		}
		
		if( !pause && getGroupMessages() )
			pop->showMessages();
	}
}

/**
 * @brief Adds a single message
 * 
 * Adds a single message sent via DCOP
 */
void MetaMonitor::addMessage( QString from, QString text )
{
	Message *m = new Message( from, text );
	messageViewer->addMessage(m);
	
	if( !getPausePopups() )
	{
		if( getGroupMessages() )
		{
			pop->addToMessageQueue( m );
			pop->showMessages();
		}
		else
			Popup::message( this, m );
	}
	delete m;
}

/**
 * @brief runs the configure window
 */
void MetaMonitor::configure()
{
	Configure dlg(this);
	int result = dlg.exec();

	if( result == QDialog::Accepted )
	{
		drawShadow = dlg.drawShadow();
		groupMessages = dlg.groupMessages();
		
		mUpdateInterval = dlg.updateInterval();
		mPopupsLifetime = dlg.popupsLifetime();
		mTimer->changeInterval((int)mUpdateInterval);
	
		QString oldFile = logFile;
		logFile = dlg.getFileName();
		if( oldFile != logFile )
		{
			delete fileMonitor;
			fileMonitor = new FileMonitor( this, logFile );
		}
		
		defaultRegExp = dlg.useDefaultRegExp();
		customRegExp = dlg.getCustomRegExp();
		
		writeConfig();
	}
}

/**
 * @brief Saves the configuration to file
 */
void MetaMonitor::writeConfig()
{
	KConfig cfg(KGlobal::dirs()->localkdedir()+"/share/config/metamonitorrc");
	
	cfg.writeEntry( "DrawShadow", drawShadow );
	cfg.writeEntry( "GroupMessages", groupMessages );
	cfg.writeEntry( "UpdateInterval", mUpdateInterval );
	cfg.writeEntry( "PopupsLifetime", mPopupsLifetime );
	cfg.writeEntry( "PausePopups", pausePopups );
	cfg.writeEntry( "LogFile", logFile );
	cfg.writeEntry( "DefaultRegExpType", defaultRegExp );
	cfg.writeEntry( "CustomRegExp", customRegExp );
}

/**
 * @brief runs the about dialog
 */
void MetaMonitor::about()
{
	KAboutApplication dlg( this );
	dlg.exec();
}

#include "metamonitor.moc"
