/***************************************************************************
 *   Copyright (C) 2005 by Krzysztof Zawadyl                               *
 *   k.zawadyl@gmail.com                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "popup.h"
#include "message.h"
#include "popupmessage.h"

#include <qlabel.h>
#include <qvbox.h>
#include <qhbox.h>
#include <qlayout.h>
#include <qpixmap.h>
#include <qptrlist.h>
#include <qevent.h>
#include <qpainter.h>
#include <qrect.h>
#include <qtooltip.h>

#include <kactivelabel.h>
#include <kdialog.h>
#include <klocale.h>

/**
 * @brief Constructor
 * 
 * Creates a popup 
 * 
 * @param parent pointer to MetaMonitor
 */
Popup::Popup( MetaMonitor *parent ) : KPassivePopup( parent ), topLayout( 0 )
{
	mIcon = parent->loadIcon( "metamonitor" );
	sshIcon = parent->loadIcon( "gpg" );
	kernelIcon = parent->loadIcon( "hwinfo" );
	
	mm = parent;
	init();	
	
	setTimeout(0);
	messageQueue.clear();
	connect( this, SIGNAL( clicked(QPoint) ), SLOT( removeMessage(QPoint) ) );
}

/**
 * @brief Destructor
 */
Popup::~Popup()
{
	messageQueue.clear();
}

/**
 * @brief Sets popup's default values
 */
void Popup::init()
{
	setPalette( QToolTip::palette() );
	// so we can drop shadow...
	setBackgroundMode( QWidget::NoBackground );
}

/**
 * @brief Paint event handler
 * 
 * To make the popup nice
 * 
 * @param e the paint event
 */
void Popup::paintEvent( QPaintEvent *e )
{
	QPainter p( this );
	
	QRect r = rect();

	r.addCoords( 0, 0, -5, -5 );
	p.setBrush( colorGroup().brush( QColorGroup::Background ) );
	
	// round popup's corners...
	int w = size().width();
	int h = size().height();
	p.drawRoundRect( r, 1500/( w ? w : 1 ), 1500/( h ? h : 1 ) );
	
	// 'round' shadow similar to qt's tooltips
	if( mm->getDrawShadow() )
	{
		p.setPen( colorGroup().shadow() );

		int w = r.width();
    	int h = r.height();
	
    	int i;
    	for( i=7; i<h; i+=2 )
        	p.drawLine( w, i, w + 5, i + 5 );
    
		for( i=w-i+h; i>6; i-=2 )
        	p.drawLine( i, h, i + 5, h + 5 );
	
		for( i=0; i<5; i+=2 )
			p.drawLine( w-2, h-2+i, w+2, h-2+i );
	}
	
	QWidget::paintEvent( e );
}

/**
 * @brief Adds a message to MessageQueue
 * 
 * This constructs a PopupMessage object from a Message and appends it to the messageQueue list
 * 
 * @param m Pointer to a message object
 */
void Popup::addToMessageQueue(Message *m)
{
	messageQueue.append( new PopupMessage( m, this ) );
}

/**
 * @brief Sets the Popup's main view
 * 
 * Creates inner part of a Popup, put's every PopupMessage from messageQueue to
 * a QVBoxLayout
 */
void Popup::setView()
{
	// first, take back our messages from topLayout to Popup
	// deleting topLayout would destroy them
	QPtrListIterator<PopupMessage> i( messageQueue );
	PopupMessage *p;
	while ( (p = i.current()) != 0 ) 
	{
		p->reparent( this, 0, p->geometry().topLeft() );
		++i;
	}
	
	// now we can recreate the layout
	delete topLayout;
	topLayout = new QVBoxLayout( this, KDialog::marginHint(), KDialog::spacingHint() );
	
	// now let's move our messages into the brand new layout :)
	QPtrListIterator<PopupMessage> it( messageQueue );
	PopupMessage *pm;
	while ( (pm = it.current()) != 0 ) 
	{
		topLayout->addWidget( pm );
		pm->startTimer();
		++it;
	}
	
	topLayout->activate();
}

/**
 * @brief Shows messages
 * 
 * Shows messages if there are any
 */
void Popup::showMessages()
{
	hide();
		
	removeOldMessages();
	
	if( messageQueue.isEmpty() )
		return;
	
	setView();
	show();
}

/**
 * @brief Removes the message located on the position
 * 
 * This handles the user's click on the single message.
 * The message is removed from the list and deleted.
 * 
 * @param pos Position of message
 */
void Popup::removeMessage(QPoint pos)
{
	QPtrListIterator<PopupMessage> it( messageQueue );
	PopupMessage *pm;
	while ( (pm = it.current()) != 0 ) 
	{
		if( pm->geometry().contains(pos) )
		{
			removeMessage( pm );
			break;
		}
		++it;
	}
}

/**
 * @brief Removes the message
 * 
 * The message is removed from the list and deleted. 
 * It shows undeleted messages once again after that.
 * 
 * @param pm PopupMessage pointer
 */
void Popup::removeMessage(PopupMessage *pm)
{
	messageQueue.remove( pm );
	delete pm;
	
	// let's show the new popup
	showMessages();
}

/**
 * @brief Removes old (expired) messages from the queue
 * 
 * Every expired message is unlinked and deleted
 */
void Popup::removeOldMessages()
{
	QPtrListIterator<PopupMessage> it( messageQueue );
	PopupMessage *pm;
	while ( (pm = it.current()) != 0 ) 
	{
		if( pm->expired )
		{
			messageQueue.remove( pm );
			delete pm;
		}
		++it;
	}
}

/**
 * @brief Static method to show single popup
 *
 * Creates the single popup with one message and shows it
 */
Popup * Popup::message( MetaMonitor *parent, Message *m )
{
	Popup *p = new Popup( parent );
	p->setTimeout( p->lifeTime() );
	p->addToMessageQueue( m );
	p->showMessages();
	return p;
}
#include "popup.moc"
 

